from django.apps import AppConfig as BaseAppConfig
from django.db.models.signals import post_migrate
from django.utils.translation import gettext_lazy as _


class SimpelPartnersConfig(BaseAppConfig):
    icon = "account-box-outline"
    default_auto_field = "django.db.models.BigAutoField"
    name = "simpel_partners"
    label = "simpel_partners"
    verbose_name = _("Partners")

    def ready(self):
        post_migrate.connect(init_app, sender=self)
        return super().ready()


def init_app(**kwargs):
    pass



# Generated by Django 3.2.13 on 2022-04-17 14:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('simpel_partners', '0002_auto_20220417_1416'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='partneraddress',
            name='address_type',
        ),
        migrations.RemoveField(
            model_name='partneraddress',
            name='primary',
        ),
        migrations.AlterField(
            model_name='partneraddress',
            name='title',
            field=models.CharField(blank=True, choices=[('Mr', 'Mr'), ('Miss', 'Miss'), ('Mrs', 'Mrs'), ('Ms', 'Ms'), ('Dr', 'Dr')], help_text='Treatment Pronouns for the customer', max_length=64, verbose_name='Title'),
        ),
    ]



from django.contrib import admin
from django.contrib.admin.templatetags.admin_urls import admin_urlname
from django.contrib.auth import get_user_model
from django.contrib.messages.storage.fallback import FallbackStorage
from django.test import RequestFactory, TestCase
from django.urls import reverse

from ..admin import PartnerAdmin
from ..models import Partner

User = get_user_model()


class DummyRequest:
    def __init__(self, user):
        self.user = user


class PartnerAdminTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.staff = User.objects.create_user(
            "staff",
            email="staff@email.com",
            password="demo_pass",
            is_staff=True,
        )
        cls.superuser = User.objects.create_superuser(
            "super",
            email="super@email.com",
            password="demo_pass",
        )

    def setUp(self):
        self.admin = PartnerAdmin(Partner, admin.site)
        self.partner = Partner(
            user=self.staff,
            name="Staff Partner",
            partner_type=Partner.ORGANIZATION,
        )
        self.partner_2 = Partner(
            user=self.superuser,
            name="Superuser Partner",
            partner_type=Partner.ORGANIZATION,
        )
        self.partner.save()
        self.partner_2.save()
        self.factory = RequestFactory()
        self.change_url = reverse(admin_urlname(self.admin.opts, "changelist"))

        staff_request = self.factory.get(self.change_url)
        staff_request.user = self.staff
        setattr(staff_request, "session", "session")
        self.staff_messages = FallbackStorage(staff_request)
        setattr(staff_request, "_messages", self.staff_messages)
        self.staff_request = staff_request

        superuser_request = self.factory.get(self.change_url)
        superuser_request.user = self.superuser
        setattr(superuser_request, "session", "session")
        self.superuser_messages = FallbackStorage(superuser_request)
        setattr(staff_request, "_messages", self.superuser_messages)
        self.superuer_request = superuser_request

        return super().setUp()

    def test_partner_admin_permissions(self):
        # Staff
        request = DummyRequest(self.staff)
        view_perm = self.admin.has_view_permission(request)
        self.assertEqual(view_perm, True)
        activate_perm = self.admin.has_activate_permission(request)
        self.assertEqual(activate_perm, False)
        verify_perm = self.admin.has_verify_permission(request)
        self.assertEqual(verify_perm, False)

        # Super User
        request = DummyRequest(self.superuser)
        view_perm = self.admin.has_view_permission(request)
        self.assertEqual(view_perm, True)
        activate_perm = self.admin.has_activate_permission(request)
        self.assertEqual(activate_perm, True)
        verify_perm = self.admin.has_verify_permission(request)
        self.assertEqual(verify_perm, True)

    def test_partner_name_method(self):
        name = self.admin.partner_name(self.partner)
        self.assertLessEqual(len(name), 60)

    def test_partner_readonly_fields(self):
        # Staff has user in readonly_fields
        request = DummyRequest(self.staff)
        readonly_fields = self.admin.get_readonly_fields(request)
        self.assertIn("user", readonly_fields)
        # Superuser has empty readonly_fields
        request = DummyRequest(self.superuser)
        readonly_fields = self.admin.get_readonly_fields(request)
        self.assertEqual(len(readonly_fields), 0)

    def test_activate_partner_action(self):
        queryset = self.admin.model.objects.all()
        queryset.update(is_active=False)

        inactive_partners = queryset.filter(is_active=False)
        self.assertEqual(inactive_partners.count(), 2)

        self.admin.activate_partner(self.staff_request, queryset)
        active_partners = self.admin.model.objects.filter(is_active=True)
        self.assertEqual(active_partners.count(), 2)

    def test_deactivate_partner_action(self):
        queryset = self.admin.model.objects.all()
        queryset.update(is_active=True)

        active_partners = queryset.filter(is_active=True)
        self.assertEqual(active_partners.count(), 2)

        self.admin.deactivate_partner(self.staff_request, queryset)
        inactive_partners = self.admin.model.objects.filter(is_active=False)
        self.assertEqual(inactive_partners.count(), 2)

    def test_verify_partner_action(self):
        queryset = self.admin.model.objects.all()
        queryset.update(is_active=True, is_verified=False)

        unverified_partners = queryset.filter(is_verified=False)
        self.assertEqual(unverified_partners.count(), 2)

        self.admin.verify_partner(self.staff_request, queryset)
        verified_partners = self.admin.model.objects.filter(is_verified=True)
        self.assertEqual(verified_partners.count(), 2)

    def test_verify_inactive_partner_action(self):
        queryset = self.admin.model.objects.all()
        queryset.update(is_active=False, is_verified=False)

        unverified_partners = queryset.filter(is_verified=False)
        self.assertEqual(unverified_partners.count(), 2)

        self.admin.verify_partner(self.staff_request, queryset)
        verified_partners = self.admin.model.objects.filter(is_verified=True)
        self.assertEqual(verified_partners.count(), 0)

    def test_deverify_partner_action(self):
        queryset = self.admin.model.objects.all()
        queryset.update(is_active=True, is_verified=True)

        verified_partners = queryset.filter(is_verified=True)
        self.assertEqual(verified_partners.count(), 2)

        self.admin.deverify_partner(self.staff_request, queryset)
        unverified_partners = self.admin.model.objects.filter(is_verified=False)
        self.assertEqual(unverified_partners.count(), 2)

    def test_activate_partner_action_view(self):
        queryset = self.admin.model.objects.all()
        queryset.update(is_active=False)
        data = {"action": "activate_partner", "_selected_action": [obj.id for obj in queryset.all()]}
        change_url = reverse(admin_urlname(self.admin.opts, "changelist"))

        self.client.login(username=self.staff.username, password="demo_pass")
        response = self.client.post(change_url, data, follow=True)
        self.client.logout()

        active_partners = self.admin.model.objects.filter(is_active=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(active_partners.count(), 0)

    def test_deactivate_partner_action_view(self):
        queryset = self.admin.model.objects.all()
        queryset.update(is_active=False)
        data = {"action": "deactivate_partner", "_selected_action": [obj.id for obj in queryset.all()]}
        change_url = reverse(admin_urlname(self.admin.opts, "changelist"))

        self.client.login(username=self.staff.username, password="demo_pass")
        response = self.client.post(change_url, data, follow=True)
        self.client.logout()

        active_partners = self.admin.model.objects.filter(is_active=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(active_partners.count(), 0)



# Generated by Django 3.2.12 on 2022-04-17 14:16

from django.db import migrations, models
import simpel_utils.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('simpel_partners', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='partner',
            name='idn',
            field=models.CharField(help_text='Identification Number', max_length=25, null=True, verbose_name='Identification Number'),
        ),
        migrations.AlterField(
            model_name='partner',
            name='tax_id',
            field=models.CharField(blank=True, help_text='Tax ID', max_length=25, null=True, verbose_name='Tax ID'),
        ),
        migrations.AlterField(
            model_name='partneraddress',
            name='line_2',
            field=simpel_utils.models.fields.NullCharField(max_length=255, verbose_name='line 2'),
        ),
        migrations.AlterField(
            model_name='partneraddress',
            name='line_3',
            field=simpel_utils.models.fields.NullCharField(max_length=255, verbose_name='line 3'),
        ),
    ]



from django.db import models


class PartnerAddressManager(models.Manager):
    def get_primary(self, partner):
        qs = self.get_queryset()
        return qs.filter(partner=partner, primary=True).first()



# Generated by Django 3.2.13 on 2022-04-20 16:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('simpel_partners', '0003_auto_20220417_1439'),
    ]

    operations = [
        migrations.AddField(
            model_name='partneraddress',
            name='address_type',
            field=models.CharField(choices=[('home', 'Home'), ('office', 'Office'), ('branch_office', 'Branch Office'), ('billing', 'Billing'), ('shipping', 'Shipping'), ('drop_shipping', 'Dropshipping'), ('deliverable', 'Deliverable'), ('else', 'Else')], default='billing', help_text='E.g. Shipping or billing', max_length=255, verbose_name='type'),
        ),
        migrations.AddField(
            model_name='partneraddress',
            name='primary',
            field=models.BooleanField(default=False),
        ),
    ]



from django.contrib.auth import get_user_model
from django.test import TestCase

from simpel_contacts.models import Country

from ..models import Partner, PartnerAddress

User = get_user_model()


class PartnerTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user_1 = User.objects.create_superuser(
            "demo_user_1",
            email="demo1@email.com",
            password="demo_pass",
        )
        cls.user_2 = User.objects.create_superuser(
            "demo_user_2",
            email="demo2@email.com",
            password="demo_pass",
        )
        cls.organization = Partner(
            user=cls.user_1,
            name="Organization Partner",
            partner_type=Partner.ORGANIZATION,
        )
        cls.organization.save()
        cls.personal = Partner(
            name="Personal Partner",
            partner_type=Partner.PERSONAL,
        )
        cls.personal.save()
        cls.partners = Partner.objects.all()

        # Partner Address
        country = Country.objects.get(iso_3166_1_a2="ID")
        address_data = dict(
            partner=cls.organization,
            title=PartnerAddress.MR,
            line_1="Street name",
            line_2="",
            line_3="",
            city="Bandar Lampung",
            province="Lampung",
            country=country,
            postcode="35223",
        )

        cls.partner_address_1 = PartnerAddress(
            address_type=PartnerAddress.SHIPPING,
            name="Partner 1",
            **address_data,
        )
        cls.partner_address_1.save()

        cls.partner_address_2 = PartnerAddress(
            address_type=PartnerAddress.BILLING,
            name="Partner 2",
            **address_data,
        )
        cls.partner_address_2.save()

        cls.partner_address_3 = PartnerAddress(
            address_type=PartnerAddress.DELIVERABLE,
            name="Partner 3",
            **address_data,
        )
        cls.partner_address_3.save()

    def tests_create_partners(self):
        self.assertEqual(str(self.organization), "Organization Partner")
        self.assertEqual(self.partners.count(), 2)
        self.assertEqual(self.partners.filter(partner_type=Partner.ORGANIZATION).count(), 1)
        self.assertEqual(self.partners.filter(partner_type=Partner.PERSONAL).count(), 1)

    def test_activate_partner(self):
        self.organization.is_active = False
        self.organization.save()
        self.organization.activate()
        self.assertEqual(self.organization.is_active, True)

    def test_deactivate_partner(self):
        self.organization.is_active = True
        self.organization.save()
        self.organization.deactivate()
        self.assertEqual(self.organization.is_active, False)

    def test_verify_active_partner(self):
        self.organization.is_active = True
        self.organization.is_verified = False
        self.organization.save()
        self.organization.verify()
        self.assertEqual(self.organization.is_verified, True)

    def test_verify_inactive_partner(self):
        self.organization.is_active = False
        self.organization.is_verified = False
        self.organization.save()
        with self.assertRaises(PermissionError):
            self.organization.verify()

    def test_deverify_verified_partner(self):
        self.organization.is_verified = True
        self.organization.save()
        self.organization.deverify()
        self.assertEqual(self.organization.is_verified, False)

    def test_deverify_unverified_partner(self):
        self.organization.is_verified = False
        self.organization.save()
        self.organization.deverify()
        self.assertEqual(self.organization.is_verified, False)

    def test_get_partner_for_user(self):
        partner_1 = Partner.get_for_user(self.user_1)
        partner_2 = Partner.get_for_user(self.user_2)
        self.assertEqual(partner_1.name, "Organization Partner")
        self.assertEqual(partner_2.user, self.user_2)
        self.assertEqual(partner_2.name, "demo_user_2")

    def test_partner_get_address(self):
        address = self.organization.get_address(address_type=PartnerAddress.HOME)
        self.assertEqual(address.name, "Partner 1")

    def test_get_partner_address_get_primary(self):
        organization_1_primary = PartnerAddress.objects.get_primary(partner=self.organization)
        personal_2_primary = PartnerAddress.objects.get_primary(partner=self.personal)
        self.assertEqual(organization_1_primary.name, "Partner 1")
        self.assertEqual(personal_2_primary, None)

    def test_get_partner_address_set_as_primary(self):
        self.partner_address_2.set_as_primary()
        primary = PartnerAddress.objects.get_primary(partner=self.organization)
        self.assertEqual(primary.id, self.partner_address_2.id)
        # Double Check
        self.partner_address_1.set_as_primary()
        primary = PartnerAddress.objects.get_primary(partner=self.organization)
        self.assertEqual(primary.id, self.partner_address_1.id)
        # make sure primary only 1 primary
        primaries = PartnerAddress.objects.filter(partner=self.organization, primary=True)
        self.assertEqual(primaries.count(), 1)

    def test_partner_address_property(self):
        address = self.organization.address
        shipping = self.organization.shipping_address
        billing = self.organization.billing_address
        deliverable = self.organization.deliverable_address
        self.assertEqual(address, self.organization.get_address())
        self.assertEqual(shipping.name, "Partner 1")
        self.assertEqual(billing.name, "Partner 2")
        self.assertEqual(deliverable.name, "Partner 3")

    def test_get_deliverable_info(self):
        deliverable = self.organization.deliverable_address
        deliverable_info = self.organization.get_deliverable_info()
        self.assertEqual(deliverable.to_dict(), deliverable_info)
        deliverable_info = self.personal.get_deliverable_info()
        self.assertEqual(dict(), deliverable_info)
